/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Martin Rexa
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import cz.xelfi.quoridor.Move;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
//import javax.xml.bind.annotation.XmlAccessType;
//import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;

/**
 *
 * @author Martin Rexa
 */

@XmlRootElement
//@XmlAccessorType(XmlAccessType.FIELD)
public class OpeningNodeView {
    @XmlAttribute
    String code;
    @XmlElementWrapper(name="whiteGames")
    @XmlElement(name="gameId")
    List<GameId> whiteGames;
    @XmlElementWrapper(name="blackGames")
    @XmlElement(name="gameId")
    List<GameId> blackGames;
    Map<Move, OpeningNodeViewEntry> entries;
    boolean empty;

    public OpeningNodeView(){
    }

    public OpeningNodeView(String code){
        this.code = code;
        entries = new HashMap<Move, OpeningNodeViewEntry>();
        whiteGames = new ArrayList<GameId>();
        blackGames = new ArrayList<GameId>();
        empty = true;
    }

    @XmlElement
    public String getNodeCode(){
        return code;
    }

    public void processGame(Move m, String code, GameId gId){
        OpeningNodeViewEntry e = entries.get(m);
        if(e == null){
            e = new OpeningNodeViewEntry(m,code);
            entries.put(m, e);
        }
        e.processGame(gId);
        empty = false;
    }

    @XmlElementWrapper(name="children")
    @XmlElement(name="item")
    public Set<OpeningNodeViewEntry> getChildren(){
        Set<OpeningNodeViewEntry> result = new HashSet<OpeningNodeViewEntry>();
        for(Map.Entry<Move, OpeningNodeViewEntry> e: entries.entrySet()){
            result.add(e.getValue());
        }
        return result;
    }


    public void addFinishedGame(GameId gId){
        if(gId.getStatus().equals(GameStatus.whiteWon))
            whiteGames.add(gId);
        if(gId.getStatus().equals(GameStatus.blackWon))
            blackGames.add(gId);
        empty = false;
    }

    @XmlAttribute
    public int getWhiteCount(){
        return whiteGames.size();
    }

    @XmlAttribute
    public int getBlackCount(){
        return blackGames.size();
    }


    @XmlAttribute
    public boolean getChildrenNotEmpty(){
        return !entries.isEmpty();
    }

    public boolean isEmpty(){
        return this.empty;
    }

}
